import bpy
import gpu
import blf
import bpy_extras
import math
from gpu_extras.batch import batch_for_shader
from mathutils import Vector, Matrix
from bpy.types import Operator, Panel, PropertyGroup
from bpy.props import PointerProperty
from bpy_extras import view3d_utils
from .lighting_gadgets_utils import update_light_transform, calculate_hit_distance, find_closest_surface_point, update_light_precise_transform

class QuickStudioLightProperties(PropertyGroup):
    """Property group for storing light hit data"""
    last_hit_location: bpy.props.FloatVectorProperty(
        name="Last Hit Location",
        description="Last position where the ray hit",
        subtype='XYZ',
        size=3
    )
    last_hit_distance: bpy.props.FloatProperty(
        name="Last Hit Distance",
        description="Distance of the last ray hit"
    )
    last_hit_normal: bpy.props.FloatVectorProperty(
        name="Last Hit Normal",
        description="Normal vector of the last ray hit surface",
        subtype='XYZ',
        size=3
    )

def draw_callback_px(self, context):
    """Draw viewport border and mode hint text"""
    try:
        # 检查是否应该绘制
        if not self.is_active or context.area != self._active_area:
            return
            
        prefs = context.preferences.addons[__package__].preferences
        
        # 获取视口区域和区域数据
        region = context.region
        rv3d = context.region_data
        
        # 设置颜色
        if self._precise_mode:
            color = prefs.precise_color
        else:
            color = prefs.reflection_color
            
        # 设置透明度
        color = (*color, prefs.border_opacity)
        text_color = (*color[:3], prefs.text_opacity)
        
        # 绘制边框
        draw_border(context, color, prefs.border_thickness)
        
        # 如果有命中点，绘制一个小点标记
        if hasattr(self, '_hit_point') and self._hit_point is not None:
            hit_2d = bpy_extras.view3d_utils.location_3d_to_region_2d(region, rv3d, self._hit_point)
            if hit_2d:
                draw_hit_point(hit_2d, prefs.text_opacity)
        
        # 准备模式文本
        if self._distance_mode:
            mode_text = "Distance Mode (G) | LMB: Confirm, RMB: Cancel"
        elif self._scale_mode:
            mode_text = "Scale Mode (S) | LMB: Confirm, RMB: Cancel"
        elif self._precise_mode:
            mode_text = "Precise Mode (Space) | Hold Shift to Slow Down"
        else:
            mode_text = "Reflection Mode | Press:\nSpace: Precise Mode\nG: Distance Mode\nS: Scale Mode\nHold Shift to Speed Up"
        
        # 计算文本位置
        font_id = 0
        text_size = prefs.text_size
        blf.size(font_id, text_size)
        
        # 获取文本尺寸
        text_lines = mode_text.split('\n')
        max_width = 0
        total_height = 0
        line_height = (blf.dimensions(font_id, "Tg")[1] * 1.5)  # 1.5倍行距
        
        for line in text_lines:
            width = blf.dimensions(font_id, line)[0]
            max_width = max(max_width, width)
            total_height += line_height
        
        # 计算文本框位置（左下角，留出边距）
        margin = 20
        text_x = margin
        text_y = margin + total_height  # 从底部向上放置文本
        
        # 绘制文本（带描边）
        current_y = text_y  # 从底部开始向上绘制每行
        for line in text_lines:
            # 绘制描边（使用多个偏移）
            offsets = ((-1, -1), (-1, 1), (1, -1), (1, 1),
                      (-1, 0), (1, 0), (0, -1), (0, 1))
            
            # 绘制黑色描边
            blf.color(font_id, 0.0, 0.0, 0.0, prefs.text_opacity)
            for offset_x, offset_y in offsets:
                blf.position(font_id, text_x + offset_x, current_y - line_height + offset_y, 0)
                blf.draw(font_id, line)
            
            # 绘制主文本
            blf.position(font_id, text_x, current_y - line_height, 0)
            blf.color(font_id, *text_color)
            blf.draw(font_id, line)
            
            current_y -= line_height
            
    except Exception as e:
        print(f"Error in draw_callback_px: {str(e)}")

def draw_border(context, color, thickness):
    """绘制视口边框"""
    width = context.area.width
    height = context.area.height
    
    shader = gpu.shader.from_builtin('UNIFORM_COLOR')
    gpu.state.blend_set('ALPHA')
    gpu.state.line_width_set(thickness)
    
    # 定义边框顶点
    vertices = (
        (0, 0), (width, 0),
        (width, 0), (width, height),
        (width, height), (0, height),
        (0, height), (0, 0)
    )
    
    batch = batch_for_shader(shader, 'LINES', {"pos": vertices})
    shader.bind()
    shader.uniform_float("color", color)
    batch.draw(shader)
    
    gpu.state.blend_set('NONE')
    gpu.state.line_width_set(1.0)

def draw_box(corners, color):
    """绘制一个填充的矩形
    corners: ((x1, y1), (x2, y2)) - 左上和右下角坐标
    """
    shader = gpu.shader.from_builtin('UNIFORM_COLOR')
    gpu.state.blend_set('ALPHA')
    
    x1, y1 = corners[0]
    x2, y2 = corners[1]
    
    vertices = (
        (x1, y1), (x2, y1),
        (x2, y2), (x1, y2)
    )
    
    batch = batch_for_shader(shader, 'TRI_FAN', {"pos": vertices})
    shader.bind()
    shader.uniform_float("color", color)
    batch.draw(shader)
    
    gpu.state.blend_set('NONE')

def draw_hit_point(position, opacity):
    """绘制命中点标记
    position: (x, y) - 屏幕坐标
    opacity: 不再使用，保持最大亮度
    """
    shader = gpu.shader.from_builtin('UNIFORM_COLOR')
    gpu.state.blend_set('ALPHA')
    
    # 定义点的参数
    radius = 4  # 点的大小
    segments = 16  # 圆形段数
    center_x, center_y = position
    
    # 创建圆形顶点
    vertices = []
    for i in range(segments):
        angle = 2 * math.pi * i / segments
        x = center_x + radius * math.cos(angle)
        y = center_y + radius * math.sin(angle)
        vertices.append((x, y))
    
    # 绘制白色填充（始终使用最大不透明度）
    batch = batch_for_shader(shader, 'TRI_FAN', {"pos": [(center_x, center_y)] + vertices})
    shader.bind()
    shader.uniform_float("color", (1.0, 1.0, 1.0, 1.0))  # 完全不透明的白色
    batch.draw(shader)
    
    # 绘制黑色边框（始终使用最大不透明度）
    vertices.append(vertices[0])  # 闭合路径
    batch = batch_for_shader(shader, 'LINE_STRIP', {"pos": vertices})
    shader.bind()
    shader.uniform_float("color", (0.0, 0.0, 0.0, 1.0))  # 完全不透明的黑色
    gpu.state.line_width_set(1.0)
    batch.draw(shader)
    
    gpu.state.blend_set('NONE')

class LIGHTING_GADGETS_OT_REFLECTION_MODE(Operator):
    bl_idname = "lighting_gadgets.reflection_mode"
    bl_label = "Reflection Mode"
    bl_description = ("Enable Reflection Mode (Shortcut: Shift + R)\n"
                     "Hold Shift to speed up in standard mode\n"
                     "Press Space to enter precise mode, hold Shift to slow down in precise mode")
    bl_options = {'REGISTER', 'UNDO'}
    
    # 类属性
    EDGE_THRESHOLD = 2
    
    @classmethod
    def poll(cls, context):
        return context.active_object and context.active_object.type == 'LIGHT'
        
    _timer = None
    is_active = False
    _initial_transform = None
    _initial_distance = None
    _last_hit_location = None
    _last_hit_normal = None
    _draw_handle = None
    _active_area = None  # Store the area where reflection mode was triggered
    _precise_mode = False  # Whether in precise control mode
    _last_mouse_pos = None  # Mouse position for precise control mode
    _hit_point = None  # Hit point
    _last_hit_distance = None  # Last hit distance
    _initial_location = None
    _initial_rotation = None
    _distance_mode = False
    _initial_distance = None
    _initial_mouse_x = None
    _scale_mode = False
    _initial_scale = None
    _initial_mouse_x = None
    
    def modal(self, context, event):
        if not context.active_object or context.active_object.type != 'LIGHT':
            self.cleanup(context, cancel=True)
            return {'CANCELLED'}

        # Only trigger redraw in active area
        if context.area == self._active_area:
            context.area.tag_redraw()

        # Handle space key for precise control mode toggle
        if event.type == 'SPACE' and event.value == 'PRESS':
            if self._last_hit_location is not None:  # Only toggle if we have a hit point
                self._precise_mode = not self._precise_mode
                self._last_mouse_pos = (event.mouse_region_x, event.mouse_region_y)
                # Force redraw to update border color
                if context.area:
                    context.area.tag_redraw()
                return {'RUNNING_MODAL'}
            return {'RUNNING_MODAL'}  # Keep current mode if no hit point

        if event.type == 'S' and event.value == 'PRESS' and not self._distance_mode and not self._scale_mode:
            # 进入缩放模式
            self._scale_mode = True
            light_obj = context.active_object
            self._initial_scale = light_obj.scale.copy()
            self._initial_mouse_x = event.mouse_region_x
            return {'RUNNING_MODAL'}

        if self._scale_mode:
            if event.type == 'MOUSEMOVE':
                # 计算鼠标在X轴上的移动距离
                delta_x = event.mouse_region_x - self._initial_mouse_x
                # 将移动距离转换为缩放因子（可以调整系数来控制灵敏度）
                scale_factor = 1.0 + (delta_x * 0.01)
                # 确保缩放不会太小
                scale_factor = max(0.1, scale_factor)
                
                # 更新灯光缩放
                light_obj = context.active_object
                light_obj.scale = self._initial_scale * scale_factor
                
                # 强制更新视图
                context.view_layer.update()
                return {'RUNNING_MODAL'}
                
            elif event.type == 'LEFTMOUSE' and event.value == 'PRESS':
                # 确认缩放并返回之前的模式
                self._scale_mode = False
                self._initial_scale = None
                self._initial_mouse_x = None
                return {'RUNNING_MODAL'}
                
            elif event.type == 'RIGHTMOUSE' and event.value == 'PRESS':
                # 取消缩放，恢复初始大小并返回之前的模式
                light_obj = context.active_object
                light_obj.scale = self._initial_scale.copy()
                context.view_layer.update()
                
                # 只退出缩放模式，保持在反射/精确模式中
                self._scale_mode = False
                self._initial_scale = None
                self._initial_mouse_x = None
                return {'RUNNING_MODAL'}
            
            # 在缩放模式下，阻止其他模式的处理
            return {'RUNNING_MODAL'}

        if event.type == 'G' and event.value == 'PRESS' and not self._distance_mode and not self._scale_mode:
            # 进入距离调整模式
            self._distance_mode = True
            light_obj = context.active_object
            self._initial_distance = (self._last_hit_location - light_obj.location).length
            self._initial_mouse_x = event.mouse_region_x
            return {'RUNNING_MODAL'}

        if self._distance_mode:
            if event.type == 'MOUSEMOVE':
                # 计算鼠标在X轴上的移动距离
                delta_x = event.mouse_region_x - self._initial_mouse_x
                # 将移动距离转换为实际距离变化（可以调整系数来控制灵敏度）
                distance_factor = 0.01
                new_distance = self._initial_distance + (delta_x * distance_factor)
                # 确保距离不会变为负值
                new_distance = max(0.1, new_distance)
                
                # 更新灯光位置
                light_obj = context.active_object
                direction = (self._last_hit_location - light_obj.location).normalized()
                light_obj.location = self._last_hit_location - direction * new_distance
                
                # 强制更新视图
                context.view_layer.update()
                return {'RUNNING_MODAL'}
                
            elif event.type == 'LEFTMOUSE' and event.value == 'PRESS':
                # 确认距离调整并返回之前的模式
                self._distance_mode = False
                self._initial_distance = None
                self._initial_mouse_x = None
                return {'RUNNING_MODAL'}
                
            elif event.type == 'RIGHTMOUSE' and event.value == 'PRESS':
                # 取消距离调整，恢复初始位置并返回之前的模式
                light_obj = context.active_object
                direction = (self._last_hit_location - light_obj.location).normalized()
                light_obj.location = self._last_hit_location - direction * self._initial_distance
                context.view_layer.update()
                
                # 只退出距离调整模式，保持在反射/精确模式中
                self._distance_mode = False
                self._initial_distance = None
                self._initial_mouse_x = None
                
                # 防止事件传递到其他处理器
                return {'RUNNING_MODAL'}
            
            # 在距离调整模式下，阻止其他模式的处理
            return {'RUNNING_MODAL'}

        if event.type == 'TIMER':
            light_obj = context.active_object
            
            if self._precise_mode:
                # Precise control mode
                if self._last_mouse_pos is None:
                    self._last_mouse_pos = (event.mouse_region_x, event.mouse_region_y)
                    return {'RUNNING_MODAL'}
                
                # Get current mouse position
                mouse_pos = (event.mouse_region_x, event.mouse_region_y)
                
                # Perform light transformation
                update_light_precise_transform(
                    context,
                    light_obj,
                    self._last_hit_location,
                    self._last_hit_normal,
                    mouse_pos,
                    self._last_mouse_pos,
                    event.shift
                )
                
                # Update last mouse position
                self._last_mouse_pos = mouse_pos
                
                context.view_layer.update()
                
            else:
                # Standard reflection mode
                coord = (event.mouse_region_x, event.mouse_region_y)
                region = context.region
                rv3d = context.region_data
                
                # Get ray origin and direction
                ray_origin = view3d_utils.region_2d_to_origin_3d(region, rv3d, coord)
                ray_direction = view3d_utils.region_2d_to_vector_3d(region, rv3d, coord)
                
                # Perform ray casting
                result = context.scene.ray_cast(
                    depsgraph=context.view_layer.depsgraph,
                    origin=ray_origin,
                    direction=ray_direction
                )
                
                if result[0]:  # If ray hits an object
                    hit_location = result[1]
                    hit_normal = result[2]
                    
                    # Set movement speed
                    if self._last_hit_location is None:
                        self._last_hit_location = hit_location.copy()
                        self._last_hit_normal = hit_normal.copy()
                    
                    # Choose interpolation factor based on Shift key
                    lerp_factor = 0.8 if event.shift else 0.15  # Shift speeds up movement
                    
                    # Interpolate between current and new positions
                    smoothed_location = self._last_hit_location.lerp(hit_location, lerp_factor)
                    smoothed_normal = self._last_hit_normal.lerp(hit_normal, lerp_factor)
                    
                    # Update last position and normal
                    self._last_hit_location = smoothed_location
                    self._last_hit_normal = smoothed_normal
                    
                    # Apply reflection calculation
                    update_light_transform(
                        context.region_data,
                        light_obj,
                        smoothed_location,
                        smoothed_normal,
                        self._initial_distance
                    )
                    
                    context.view_layer.update()
                    
                    # Update hit point
                    self._hit_point = hit_location
                    
        elif event.type == 'MOUSEMOVE':
            if self._precise_mode:
                # 获取当前视口区域
                region = context.region
                region_width = region.width
                region_height = region.height
                
                # 获取鼠标在视口中的局部坐标
                local_x = event.mouse_region_x
                local_y = event.mouse_region_y
                
                # 记录原始位置用于计算偏移
                original_x = local_x
                original_y = local_y
                
                # 检测边界并计算新位置
                wrap_x = False
                new_x = local_x
                if local_x <= self.EDGE_THRESHOLD:
                    new_x = region_width - self.EDGE_THRESHOLD
                    wrap_x = True
                elif local_x >= region_width - self.EDGE_THRESHOLD:
                    new_x = self.EDGE_THRESHOLD
                    wrap_x = True
                    
                wrap_y = False
                new_y = local_y
                if local_y <= self.EDGE_THRESHOLD:
                    new_y = region_height - self.EDGE_THRESHOLD
                    wrap_y = True
                elif local_y >= region_height - self.EDGE_THRESHOLD:
                    new_y = self.EDGE_THRESHOLD
                    wrap_y = True
                
                # 如果需要包裹，执行鼠标位置重置
                if wrap_x or wrap_y:
                    # 计算实际移动差值
                    if wrap_x:
                        dx = new_x - original_x
                        if original_x <= self.EDGE_THRESHOLD:
                            dx = -(region_width - 2 * self.EDGE_THRESHOLD)
                        else:
                            dx = region_width - 2 * self.EDGE_THRESHOLD
                    else:
                        dx = 0
                        
                    if wrap_y:
                        dy = new_y - original_y
                        if original_y <= self.EDGE_THRESHOLD:
                            dy = -(region_height - 2 * self.EDGE_THRESHOLD)
                        else:
                            dy = region_height - 2 * self.EDGE_THRESHOLD
                    else:
                        dy = 0
                    
                    # 更新鼠标位置
                    context.window.cursor_warp(
                        context.region.x + new_x,
                        context.region.y + new_y
                    )
                    
                    # 更新last_mouse_pos以保持运动连续性
                    self._last_mouse_pos = (new_x, new_y)
                    
                    # 应用移动
                    current_mouse = (new_x, new_y)
                    update_light_precise_transform(
                        context,
                        context.active_object,
                        self._last_hit_location,
                        self._last_hit_normal,
                        current_mouse,
                        self._last_mouse_pos,
                        event.shift
                    )
                    
                else:
                    # 正常的鼠标移动处理
                    current_mouse = (event.mouse_region_x, event.mouse_region_y)
                    update_light_precise_transform(
                        context,
                        context.active_object,
                        self._last_hit_location,
                        self._last_hit_normal,
                        current_mouse,
                        self._last_mouse_pos,
                        event.shift
                    )
                    self._last_mouse_pos = current_mouse
            else:
                # 非精确模式下的原有处理逻辑
                pass
        
        elif event.type == 'LEFTMOUSE' and event.value == 'PRESS':
            # 确认时保存落点数据
            if self._last_hit_location is not None:
                light = context.active_object
                # 保存落点数据到自定义属性
                props = light.quick_studio_light
                props.last_hit_location = self._last_hit_location.copy()
                props.last_hit_distance = self._last_hit_distance
                props.last_hit_normal = self._last_hit_normal.copy()
                
                # 清理绘制
                if self._timer is not None:
                    context.window_manager.event_timer_remove(self._timer)
                bpy.types.SpaceView3D.draw_handler_remove(self._draw_handle, 'WINDOW')
                context.area.tag_redraw()
                
                return {'FINISHED'}
            return {'CANCELLED'}
            
        elif event.type == 'RIGHTMOUSE' and event.value == 'PRESS' and not self._distance_mode and not self._scale_mode:
            # 取消时恢复初始状态，不保存落点
            light = context.active_object
            light.location = self._initial_location
            light.rotation_euler = self._initial_rotation
            
            # 清理
            if self._timer:
                context.window_manager.event_timer_remove(self._timer)
            bpy.types.SpaceView3D.draw_handler_remove(self._draw_handle, 'WINDOW')
            context.area.tag_redraw()
            return {'CANCELLED'}

        return {'RUNNING_MODAL'}

    def invoke(self, context, event):
        if context.area.type != 'VIEW_3D':
            self.report({'WARNING'}, "Use this tool in the 3D view")
            return {'CANCELLED'}
            
        if not self.poll(context):
            self.report({'ERROR'}, "Select a light object first")
            return {'CANCELLED'}

        if self.is_active:
            self.cleanup(context, cancel=True)
            return {'CANCELLED'}
        
        light_obj = context.active_object
        
        # Save initial state
        self._initial_location = light_obj.location.copy()
        self._initial_rotation = light_obj.rotation_euler.copy()
        
        # Try to find the closest surface point to the current light direction
        hit_location, hit_normal, distance = find_closest_surface_point(context, light_obj)
        
        if hit_location is not None:
            # If a surface point is found, use the actual distance
            self._initial_distance = distance
        else:
            # If no surface point is found, use the distance to the world center
            self._initial_distance = light_obj.location.length
        
        # Load last hit point data
        if not self.load_last_hit_data(light_obj):
            # If no history data, use the current hit point
            if hit_location is not None:
                self._hit_point = hit_location
                self._last_hit_location = hit_location
                self._last_hit_normal = hit_normal
                self._last_hit_distance = distance
        
        # Set timer
        wm = context.window_manager
        self._timer = wm.event_timer_add(0.016, window=context.window)
        
        # Store current active area
        self._active_area = context.area
        self.is_active = True
        
        # Add drawing callback
        args = (self, context)
        self._draw_handle = bpy.types.SpaceView3D.draw_handler_add(
            draw_callback_px, args, 'WINDOW', 'POST_PIXEL'
        )
        
        context.area.tag_redraw()
        
        wm.modal_handler_add(self)
        return {'RUNNING_MODAL'}

    def load_last_hit_data(self, light):
        """Load last hit point data from the light object"""
        try:
            if hasattr(light, "quick_studio_light"):
                props = light.quick_studio_light
                if props.last_hit_location and props.last_hit_normal is not None:
                    self._last_hit_location = Vector(props.last_hit_location)
                    self._last_hit_normal = Vector(props.last_hit_normal)
                    self._last_hit_distance = float(props.last_hit_distance)
                    self._hit_point = self._last_hit_location
                    return True
            return False
        except AttributeError as e:
            print(f"Error accessing light properties: {e}")
            return False
        except ValueError as e:
            print(f"Error converting property values: {e}")
            return False
        except Exception as e:
            print(f"Unexpected error in load_last_hit_data: {e}")
            return False

    def cleanup(self, context, cancel):
        if self._timer:
            context.window_manager.event_timer_remove(self._timer)
            self._timer = None
            
        if self._draw_handle:
            bpy.types.SpaceView3D.draw_handler_remove(self._draw_handle, 'WINDOW')
            self._draw_handle = None
            
        if cancel:
            # Restore initial state
            context.active_object.location = self._initial_location
            context.active_object.rotation_euler = self._initial_rotation
            
        self.is_active = False
        context.area.tag_redraw()

class LIGHTING_GADGETS_PT_QUICK_PANEL(Panel):
    bl_label = "Light Settings"
    bl_idname = "LIGHTING_GADGETS_PT_quick_panel"
    bl_space_type = 'VIEW_3D'
    bl_region_type = 'WINDOW'
    bl_ui_units_x = 12

    @classmethod
    def poll(cls, context):
        return context.active_object and context.active_object.type == 'LIGHT'

    def find_connected_nodes(self, output_node, connected_nodes=None):
        """递归查找与输出节点相连的所有节点"""
        if connected_nodes is None:
            connected_nodes = set()
            
        # 如果节点已经处理过，直接返回
        if output_node in connected_nodes:
            return connected_nodes
            
        connected_nodes.add(output_node)
        
        # 遍历所有输入端口
        for input in output_node.inputs:
            # 检查输入端口是否有连接
            if input.links:
                # 获取连接的来源节点
                from_node = input.links[0].from_node
                # 递归处理来源节点
                self.find_connected_nodes(from_node, connected_nodes)
                
        return connected_nodes

    def draw_node_group_inputs(self, layout, node):
        """Draw input parameters for a node group"""
        if not node.node_tree:
            return
            
        # 检查节点是否有任何未连接的输入端口
        has_unconnected_inputs = False
        for input in node.inputs:
            if not input.links and not input.hide:
                # 检查是否有任何可调整的属性
                if (hasattr(input, "default_value") or 
                    hasattr(input, "default_value_raw") or
                    input.type in {'VECTOR', 'NORMAL', 'RGBA', 'BOOLEAN'}):
                    has_unconnected_inputs = True
                    break
                
        # 如果没有未连接的输入端口，不显示这个节点组
        if not has_unconnected_inputs:
            return
            
        # Create a box for the node group
        box = layout.box()
        row = box.row()
        row.label(text=node.name)
        row.prop(node, "mute", text="", icon='HIDE_OFF' if not node.mute else 'HIDE_ON', emboss=False)
        
        # Draw input parameters
        for input in node.inputs:
            # Skip connected or hidden inputs
            if input.links or input.hide:
                continue

            # Create row for each input
            row = box.row(align=True)
            
            # 处理不同类型的输入
            if input.type == 'BOOLEAN':
                # 布尔值输入
                if hasattr(input, "default_value"):
                    row.prop(input, "default_value", text=input.name)
                    
            elif input.type == 'VECTOR' or input.type == 'NORMAL':
                # 为矢量和法线类型创建一个子布局
                row.label(text=input.name)
                if hasattr(input, "default_value"):
                    sub = row.column(align=True)
                    sub.prop(input, "default_value", text="")
                elif hasattr(input, "default_value_raw"):
                    # 某些节点使用 default_value_raw
                    sub = row.column(align=True)
                    sub.prop(input, "default_value_raw", text="")
                
            elif input.type == 'RGBA':
                # 颜色输入
                row.label(text=input.name)
                if hasattr(input, "default_value"):
                    row.prop(input, "default_value", text="")
                
            elif hasattr(input, "default_value"):
                # 处理其他类型的输入
                row.label(text=input.name)
                
                if isinstance(input.default_value, float):
                    # 数值输入
                    row.prop(input, "default_value", text="")
                elif isinstance(input.default_value, (tuple, list)):
                    if len(input.default_value) == 3:  # Color
                        row.prop(input, "default_value", text="")
                    elif len(input.default_value) == 2:  # Vector2
                        split = row.split(factor=0.5)
                        split.prop(input, "default_value", text="", index=0)
                        split.prop(input, "default_value", text="", index=1)
                    elif len(input.default_value) == 3:  # Vector3
                        split = row.split(factor=0.33)
                        split.prop(input, "default_value", text="", index=0)
                        split = split.split(factor=0.5)
                        split.prop(input, "default_value", text="", index=1)
                        split.prop(input, "default_value", text="", index=2)

    def draw(self, context):
        layout = self.layout
        light = context.active_object.data
        
        # Light type
        row = layout.row()
        row.prop(light, "type", expand=True)

        # Common properties
        col = layout.column()
        col.prop(light, "energy")
        col.prop(light, "color")
        col.prop(light, "spread")
        
        # Specific properties based on light type
        if light.type == 'POINT':
            col.prop(light, "shadow_soft_size", text="Radius")
        elif light.type == 'SPOT':
            col.prop(light, "shadow_soft_size", text="Radius")
            col.prop(light, "spot_size")
            col.prop(light, "spot_blend")
        elif light.type == 'SUN':
            col.prop(light, "angle")
        elif light.type == 'AREA':
            col.prop(light, "shape")
            sub = col.column(align=True)
            if light.shape in {'SQUARE', 'DISK'}:
                sub.prop(light, "size")
            else:
                sub.prop(light, "size", text="Size X")
                sub.prop(light, "size_y", text="Size Y")
        
        # Node groups section
        if light.use_nodes:
            # 找到 Light Output 节点
            output_node = None
            for node in light.node_tree.nodes:
                if node.type == 'OUTPUT_LIGHT':
                    output_node = node
                    break
                    
            if output_node:
                # 获取所有与输出节点相连的节点
                connected_nodes = self.find_connected_nodes(output_node)
                
                # 检查是否有节点组需要显示
                has_node_groups = False
                for node in connected_nodes:
                    if node.type == 'GROUP':
                        # 检查是否有未连接的输入端口
                        for input in node.inputs:
                            if not input.links and hasattr(input, "default_value") and not input.hide:
                                has_node_groups = True
                                break
                        if has_node_groups:
                            break
                
                if has_node_groups:
                    layout.separator()
                    layout.label(text="Node Groups")
                    
                    # 只显示相连节点中的节点组
                    for node in connected_nodes:
                        if node.type == 'GROUP':
                            self.draw_node_group_inputs(layout, node)

class LIGHTING_GADGETS_OT_QUICK_PANEL(Operator):
    bl_idname = "lighting_gadgets.quick_panel"
    bl_label = "Light Settings"

    def invoke(self, context, event):
        # 检查是否处于物体模式
        if context.mode != 'OBJECT':
            return {'PASS_THROUGH'}  # 在非物体模式下传递事件，不拦截快捷键
            
        # 检查是否选择了灯光对象
        if not context.active_object or context.active_object.type != 'LIGHT':
            self.report({'ERROR'}, "Please select a light object")
            return {'CANCELLED'}

        bpy.ops.wm.call_panel(name="LIGHTING_GADGETS_PT_quick_panel")
        return {'FINISHED'}

def register():
    """Register the addon operators and UI"""
    bpy.utils.register_class(LIGHTING_GADGETS_OT_REFLECTION_MODE)
    bpy.utils.register_class(LIGHTING_GADGETS_PT_QUICK_PANEL)
    bpy.utils.register_class(LIGHTING_GADGETS_OT_QUICK_PANEL)
    bpy.types.Light.quick_studio_light = PointerProperty(type=QuickStudioLightProperties)

def unregister():
    """Unregister the addon operators and UI"""
    bpy.utils.unregister_class(LIGHTING_GADGETS_OT_QUICK_PANEL)
    bpy.utils.unregister_class(LIGHTING_GADGETS_PT_QUICK_PANEL)
    bpy.utils.unregister_class(LIGHTING_GADGETS_OT_REFLECTION_MODE)
    del bpy.types.Light.quick_studio_light

if __name__ == "__main__":
    register()