import bpy
import gpu
import blf
import math
from mathutils import Vector, Matrix
from gpu_extras.batch import batch_for_shader
import mathutils
from math import radians, sqrt, sin, cos, pi, atan2, acos

def getCameraPosition(regionView3D):
    """Get camera position in world space"""
    if regionView3D.is_perspective:
        return regionView3D.view_matrix.inverted().translation
    else:
        return regionView3D.view_matrix.inverted().translation

def getCameraViewDirection(regionView3D):
    """Get camera view direction"""
    return regionView3D.view_rotation @ mathutils.Vector((0.0, 0.0, -1.0))

def get_view_info(region_data):
    """Get view information"""
    view_matrix = region_data.view_matrix
    view_dir = view_matrix.to_3x3().inverted() @ Vector((0.0, 0.0, -1.0))
    return view_dir

def calculate_light_orientation(direction):
    """Calculate light orientation
    
    Args:
        direction: Direction vector
        
    Returns:
        Euler rotation
    """
    # Use quaternion to calculate rotation, making light face target point
    rotation = direction.to_track_quat('-Z', 'Y')
    return rotation.to_euler()

def lookAtRotation(vec):
    """Calculate rotation to look at a vector
    
    Args:
        vec: Target vector
        
    Returns:
        Euler rotation
    """
    # Create euler rotation
    rot = mathutils.Vector((0, 0, 1)).rotation_difference(vec).to_euler()
    return rot

def calculate_hit_distance(light_obj, hit_location):
    """Calculate distance from light to hit point
    
    Args:
        light_obj: Light object
        hit_location: Hit point location
        
    Returns:
        Distance
    """
    return (hit_location - light_obj.location).length

def find_closest_surface_point(context, light_obj):
    """Find closest surface point in light's forward direction
    
    Args:
        context: Current context
        light_obj: Light object
        
    Returns:
        tuple: (hit_location, hit_normal, distance) or (None, None, None)
    """
    # Get light direction (-Z axis)
    direction = light_obj.matrix_world.to_3x3() @ Vector((0.0, 0.0, -1.0))
    direction.normalize()
    
    # Cast ray from light position
    result = context.scene.ray_cast(
        depsgraph=context.view_layer.depsgraph,
        origin=light_obj.location,
        direction=direction
    )
    
    if result[0]:  # If ray hits an object
        return result[1], result[2], calculate_hit_distance(light_obj, result[1])
    return None, None, None

def update_light_transform(activeRegion3D, lightObject, hitLocation, hitNormal, initial_distance=None):
    """Update light transformation
    
    Args:
        activeRegion3D: Current 3D view region
        lightObject: Light object
        hitLocation: Hit point location
        hitNormal: Hit point normal
        initial_distance: Distance to maintain, if None uses current distance from light to hit point
    """
    # Get current view direction
    view_dir = get_view_info(activeRegion3D)
    
    # Calculate reflection direction
    # Using formula: R = V - 2(V·N)N, where V is view direction, N is surface normal
    dot_product = view_dir.dot(hitNormal)
    reflection_dir = view_dir - (2 * dot_product * hitNormal)
    reflection_dir.normalize()
    
    # If no distance specified, use current distance from light to hit point
    if initial_distance is None:
        initial_distance = calculate_hit_distance(lightObject, hitLocation)
    
    # Set light position
    lightObject.location = hitLocation + reflection_dir * initial_distance
    
    # Calculate direction vector from light position to hit point
    to_target = (hitLocation - lightObject.location).normalized()
    
    # Set light rotation to face hit point
    lightObject.rotation_euler = calculate_light_orientation(to_target)

def lerp(a, b, t):
    """Linear interpolation
    
    Args:
        a: Start value
        b: Target value
        t: Interpolation factor (0-1)
        
    Returns:
        Interpolated value
    """
    return a + (b - a) * t

def lerp_angle(a, b, t):
    """Angle interpolation, handles angle wrapping
    
    Args:
        a: Start angle
        b: Target angle
        t: Interpolation factor (0-1)
        
    Returns:
        Interpolated angle
    """
    # Ensure angle difference is between -pi and pi
    diff = (b - a + pi) % (2 * pi) - pi
    return a + diff * t

def cartesian_to_spherical(v):
    """Convert Cartesian coordinates to spherical coordinates
    
    Args:
        v: Vector in Cartesian coordinates
        
    Returns:
        tuple: (r, theta, phi)
        r: radius
        theta: azimuthal angle (in horizontal plane, 0-2π)
        phi: polar angle (from z-axis, 0-π)
    """
    x, y, z = v
    r = sqrt(x*x + y*y + z*z)
    if r == 0:
        return 0, 0, 0
    theta = atan2(y, x)  # azimuthal angle
    phi = acos(z/r)      # polar angle
    return r, theta, phi

def spherical_to_cartesian(r, theta, phi):
    """Convert spherical coordinates to Cartesian coordinates
    
    Args:
        r: radius
        theta: azimuthal angle
        phi: polar angle
        
    Returns:
        Vector in Cartesian coordinates
    """
    x = r * sin(phi) * cos(theta)
    y = r * sin(phi) * sin(theta)
    z = r * cos(phi)
    return Vector((x, y, z))

def update_light_precise_transform(context, light_obj, hit_location, hit_normal, current_mouse, last_mouse, is_slow=False):
    """Update light position in precise control mode using spherical coordinates
    
    Args:
        context: Current context
        light_obj: Light object
        hit_location: Hit point location (rotation center)
        hit_normal: Hit point normal
        current_mouse: Current mouse position (x, y)
        last_mouse: Last mouse position (x, y)
        is_slow: Whether to use slow mode
    """
    # Get region dimensions
    region = context.region
    width = region.width
    height = region.height
    
    # Calculate mouse movement delta (considering wrapping)
    def calculate_delta(current, last, size):
        delta = current - last
        if abs(delta) > size / 2:
            if delta > 0:
                delta = delta - size
            else:
                delta = size + delta
        return delta
    
    # Calculate X and Y movement
    delta_x = calculate_delta(current_mouse[0], last_mouse[0], width)
    delta_y = calculate_delta(current_mouse[1], last_mouse[1], height)
    
    # Adjust rotation speed (consistent with reflection mode)
    speed_factor = 0.3 if is_slow else 1.0
    delta_x *= speed_factor / width
    delta_y *= speed_factor / height
    
    # Get current light vector relative to hit point
    to_light = light_obj.location - hit_location
    
    # Convert to spherical coordinates
    r, theta, phi = cartesian_to_spherical(to_light)
    
    # Update angles using smooth interpolation
    # Horizontal rotation (azimuthal angle)
    target_theta = theta + delta_x * 2 * pi  # Map movement to full circle
    theta = lerp_angle(theta, target_theta, 0.5)
    
    # Vertical rotation (polar angle)
    target_phi = phi - delta_y * pi  # Map movement to half circle
    phi = lerp(phi, target_phi, 0.5)
    
    # Limit angle ranges
    # Azimuthal angle wraps (0-2π)
    theta = theta % (2 * pi)
    # Polar angle is limited (0-π)
    phi = max(0.001, min(pi - 0.001, phi))  # Avoid singularity at poles
    
    # Convert back to Cartesian coordinates
    new_pos = spherical_to_cartesian(r, theta, phi)
    
    # Smoothly interpolate position
    target_location = hit_location + new_pos
    light_obj.location = light_obj.location.lerp(target_location, 0.5)
    
    # Ensure light always faces hit point
    direction = hit_location - light_obj.location
    rot_quat = direction.to_track_quat('-Z', 'Y')
    # Smoothly interpolate rotation
    current_quat = light_obj.rotation_euler.to_quaternion()
    new_quat = current_quat.slerp(rot_quat, 0.5)
    light_obj.rotation_euler = new_quat.to_euler()
    
    # Update view
    context.view_layer.update()
